"""Tests for feed entry."""

import datetime
from unittest import mock

from georss_client import FeedEntry


def test_simple_feed_entry():
    """Test feed entry behaviour."""
    feed_entry = FeedEntry(None, None)
    assert repr(feed_entry) == "<FeedEntry(id=None)>"
    assert feed_entry.geometry is None
    assert feed_entry.coordinates is None
    assert feed_entry.title is None
    assert feed_entry.category is None
    assert feed_entry.attribution is None
    assert feed_entry.description is None
    assert feed_entry.published is None
    assert feed_entry.updated is None
    assert (
        feed_entry._search_in_external_id(r"External ID (?P<custom_attribute>.+)$")  # noqa: SLF001
        is None
    )
    assert feed_entry._search_in_title(r"Title (?P<custom_attribute>.+)$") is None  # noqa: SLF001
    assert (
        feed_entry._search_in_description(r"Description (?P<custom_attribute>.+)$")  # noqa: SLF001
        is None
    )


def test_feed_entry_search_in_attributes():
    """Test feed entry behaviour."""
    rss_entry = mock.MagicMock()
    type(rss_entry).guid = mock.PropertyMock(return_value="Test 123")
    type(rss_entry).title = mock.PropertyMock(return_value="Title 123")
    type(rss_entry).description = mock.PropertyMock(return_value="Description 123")
    type(rss_entry).category = mock.PropertyMock(
        return_value=["Category 1", "Category 2"]
    )
    updated = datetime.datetime(2019, 4, 1, 8, 30, tzinfo=datetime.timezone.utc)
    type(rss_entry).updated_date = mock.PropertyMock(return_value=updated)

    feed_entry = FeedEntry(None, rss_entry)
    assert repr(feed_entry) == "<FeedEntry(id=Test 123)>"

    assert (
        feed_entry._search_in_external_id(r"Test (?P<custom_attribute>.+)$")  # noqa: SLF001
        == "123"
    )
    assert feed_entry._search_in_title(r"Title (?P<custom_attribute>.+)$") == "123"  # noqa: SLF001
    assert (
        feed_entry._search_in_description(r"Description (?P<custom_attribute>.+)$")  # noqa: SLF001
        == "123"
    )
    assert feed_entry.category == "Category 1"
    assert feed_entry.description == "Description 123"
    assert feed_entry.updated == updated



"""Tests for georss-client library."""

from georss_client.feed import GeoRssFeed
from georss_client.feed_entry import FeedEntry
from georss_client.xml_parser.feed_item import FeedItem


class MockGeoRssFeed(GeoRssFeed):
    """Mock GeoRSS feed."""

    def _new_entry(
        self,
        home_coordinates: tuple[float, float],
        rss_entry: FeedItem,
        global_data: dict,
    ):
        """Generate a new entry."""
        return FeedEntry(home_coordinates, rss_entry)



"""Test geometries."""

from georss_client.xml_parser.geometry import Point, Polygon


def test_point():
    """Test point."""
    point = Point(-37.1234, 149.2345)
    assert point.latitude == -37.1234
    assert point.longitude == 149.2345
    assert repr(point) == "<Point(latitude=-37.1234, longitude=149.2345)>"


def test_point_equality():
    """Test points."""
    point1 = Point(10.0, 15.0)
    point2 = Point(10.0, 15.0)
    assert point1 == point2


def test_polygon():
    """Test polygon."""
    polygon = Polygon(
        [
            Point(-30.1, 150.1),
            Point(-30.2, 150.2),
            Point(-30.4, 150.4),
            Point(-30.8, 150.8),
            Point(-30.1, 150.1),
        ]
    )
    assert len(polygon.points) == 5
    assert polygon.centroid.latitude == -30.32
    assert polygon.centroid.longitude == 150.32
    assert (
        repr(polygon) == "<Polygon(centroid="
        "<Point(latitude=-30.32, longitude=150.32)>)>"
    )


def test_polygon_equality():
    """Test points."""
    polygon1 = Polygon(
        [
            Point(30.0, 30.0),
            Point(30.0, 35.0),
            Point(35.0, 35.0),
            Point(35.0, 30.0),
            Point(30.0, 30.0),
        ]
    )
    polygon2 = Polygon(
        [
            Point(30.0, 30.0),
            Point(30.0, 35.0),
            Point(35.0, 35.0),
            Point(35.0, 30.0),
            Point(30.0, 30.0),
        ]
    )
    assert polygon1 == polygon2



"""GeoRSS feed image."""

from __future__ import annotations

from georss_client.consts import XML_TAG_HEIGHT, XML_TAG_URL, XML_TAG_WIDTH
from georss_client.xml_parser.feed_dict_source import FeedDictSource


class FeedImage(FeedDictSource):
    """Represents a feed image."""

    @property
    def url(self) -> str | None:
        """Return the url of this feed image."""
        return self._attribute([XML_TAG_URL])

    @property
    def height(self) -> int | None:
        """Return the height of this feed image."""
        return self._attribute([XML_TAG_HEIGHT])

    @property
    def width(self) -> int | None:
        """Return the width of this feed image."""
        return self._attribute([XML_TAG_WIDTH])



"""Base class for GeoRSS services."""

from georss_client.consts import ATTR_ATTRIBUTION  # noqa: F401

from .consts import (  # noqa: F401
    CUSTOM_ATTRIBUTE,
    UPDATE_ERROR,
    UPDATE_OK,
    UPDATE_OK_NO_DATA,
)
from .feed import GeoRssFeed  # noqa: F401
from .feed_entry import FeedEntry  # noqa: F401



"""GeoRSS feed dict source."""

from __future__ import annotations

from typing import Optional

from georss_client.consts import (
    XML_ATTR_HREF,
    XML_CDATA,
    XML_TAG_CONTENT,
    XML_TAG_DESCRIPTION,
    XML_TAG_LINK,
    XML_TAG_SUMMARY,
    XML_TAG_TITLE,
)


class FeedDictSource:
    """Represents a subset of a feed based on a dict."""

    def __init__(self, source: dict):
        """Initialise feed."""
        self._source: dict = source

    def __repr__(self):
        """Return string representation of this feed item."""
        return f"<{self.__class__.__name__}({self.link})>"

    def _attribute(self, names: list[str]) -> Optional:
        """Get an attribute from this feed or feed item."""
        if self._source and names:
            # Try each name, and return the first value that is not None.
            for name in names:
                value = self._source.get(name, None)
                if value:
                    return value
        return None

    def _attribute_with_text(self, names: list[str]) -> Optional:
        """Get an attribute with text from this feed or feed item."""
        value = self._attribute(names)
        if value and isinstance(value, dict) and XML_CDATA in value:
            # <tag attr="/some.uri">Value</tag>
            value = value.get(XML_CDATA)
        return value

    @staticmethod
    def _attribute_in_structure(obj, keys: list[str]) -> Optional:
        """Return the attribute found under the chain of keys."""
        key = keys.pop(0)
        if key in obj:
            return (
                FeedDictSource._attribute_in_structure(obj[key], keys)
                if keys
                else obj[key]
            )
        return None

    @property
    def title(self) -> str | None:
        """Return the title of this feed or feed item."""
        return self._attribute_with_text([XML_TAG_TITLE])

    @property
    def description(self) -> str | None:
        """Return the description of this feed or feed item."""
        return self._attribute_with_text(
            [XML_TAG_DESCRIPTION, XML_TAG_SUMMARY, XML_TAG_CONTENT]
        )

    @property
    def summary(self) -> str | None:
        """Return the summary of this feed or feed item."""
        return self.description

    @property
    def content(self) -> str | None:
        """Return the content of this feed or feed item."""
        return self.description

    @property
    def link(self) -> str | None:
        """Return the link of this feed or feed item."""
        link = self._attribute([XML_TAG_LINK])
        if link and XML_ATTR_HREF in link:
            link = link.get(XML_ATTR_HREF)
        return link

    def get_additional_attribute(self, name: str) -> Optional:
        """Get an additional attribute not provided as property."""
        return self._attribute([name])



"""Constants.

Constants for feeds and feed entries.
"""

from __future__ import annotations

from typing import Final

ATTR_ATTRIBUTION: Final = "attribution"
CUSTOM_ATTRIBUTE: Final = "custom_attribute"

XML_ATTR_HREF: Final = "@href"
XML_ATTR_TERM: Final = "@term"

XML_CDATA: Final = "#text"

XML_TAG_AUTHOR: Final = "author"
XML_TAG_CATEGORY: Final = "category"
XML_TAG_CHANNEL: Final = "channel"
XML_TAG_CONTENT: Final = "content"
XML_TAG_CONTRIBUTOR: Final = "contributor"
XML_TAG_COPYRIGHT: Final = "copyright"
XML_TAG_DC_DATE: Final = "dc:date"
XML_TAG_DESCRIPTION: Final = "description"
XML_TAG_DOCS: Final = "docs"
XML_TAG_ENTRY: Final = "entry"
XML_TAG_FEED: Final = "feed"
XML_TAG_GENERATOR: Final = "generator"
XML_TAG_GEO_LAT: Final = "geo:lat"
XML_TAG_GEO_LONG: Final = "geo:long"
XML_TAG_GEO_POINT: Final = "geo:Point"
XML_TAG_GEORSS_POINT: Final = "georss:point"
XML_TAG_GEORSS_POLYGON: Final = "georss:polygon"
XML_TAG_GEORSS_WHERE: Final = "georss:where"
XML_TAG_GML_EXTERIOR: Final = "gml:exterior"
XML_TAG_GML_LINEAR_RING: Final = "gml:LinearRing"
XML_TAG_GML_POINT: Final = "gml:Point"
XML_TAG_GML_POLYGON: Final = "gml:Polygon"
XML_TAG_GML_POS: Final = "gml:pos"
XML_TAG_GML_POS_LIST: Final = "gml:posList"
XML_TAG_GUID: Final = "guid"
XML_TAG_HEIGHT: Final = "height"
XML_TAG_ID: Final = "id"
XML_TAG_IMAGE: Final = "image"
XML_TAG_ITEM: Final = "item"
XML_TAG_LANGUAGE: Final = "language"
XML_TAG_LAST_BUILD_DATE: Final = "lastBuildDate"
XML_TAG_LINK: Final = "link"
XML_TAG_MANAGING_EDITOR: Final = "managingEditor"
XML_TAG_NAME: Final = "name"
XML_TAG_PUB_DATE: Final = "pubDate"
XML_TAG_PUBLISHED: Final = "published"
XML_TAG_RIGHTS: Final = "rights"
XML_TAG_RSS: Final = "rss"
XML_TAG_SOURCE: Final = "source"
XML_TAG_SUBTITLE: Final = "subtitle"
XML_TAG_SUMMARY: Final = "summary"
XML_TAG_TITLE: Final = "title"
XML_TAG_TTL: Final = "ttl"
XML_TAG_UPDATED: Final = "updated"
XML_TAG_URL: Final = "url"
XML_TAG_WIDTH: Final = "width"

UPDATE_OK: Final = "OK"
UPDATE_OK_NO_DATA: Final = "OK_NO_DATA"
UPDATE_ERROR: Final = "ERROR"



"""GeoRSS Feed."""

from __future__ import annotations

import codecs
from datetime import datetime
import logging

import requests

from .consts import ATTR_ATTRIBUTION, UPDATE_ERROR, UPDATE_OK, UPDATE_OK_NO_DATA
from .xml_parser import Feed, XmlParser
from .xml_parser.feed_item import FeedItem

_LOGGER = logging.getLogger(__name__)


class GeoRssFeed:
    """GeoRSS feed base class."""

    def __init__(
        self,
        home_coordinates: tuple[float, float],
        url: str,
        filter_radius: float | None = None,
        filter_categories: list[str] | None = None,
    ):
        """Initialise this service."""
        self._home_coordinates: tuple[float, float] = home_coordinates
        self._filter_radius: float | None = filter_radius
        self._filter_categories: list[str] | None = filter_categories
        self._url: str = url
        self._request = requests.Request(method="GET", url=url).prepare()
        self._last_timestamp: datetime | None = None

    def __repr__(self):
        """Return string representation of this feed."""
        return f"<{self.__class__.__name__}(home={self._home_coordinates}, url={self._url}, radius={self._filter_radius}, categories={self._filter_categories})>"

    def _new_entry(
        self,
        home_coordinates: tuple[float, float],
        rss_entry: FeedItem,
        global_data: dict,
    ):
        """Generate a new entry."""

    def _additional_namespaces(self):
        """Provide additional namespaces, relevant for this feed."""

    def update(self):
        """Update from external source and return filtered entries."""
        status, data = self._fetch()
        if status == UPDATE_OK:
            if data:
                global_data = self._extract_from_feed(data)
                # Extract data from feed entries.
                entries: list = [
                    self._new_entry(self._home_coordinates, rss_entry, global_data)
                    for rss_entry in data.entries
                ]
                filtered_entries = self._filter_entries(entries)
                self._last_timestamp = self._extract_last_timestamp(filtered_entries)
                return UPDATE_OK, filtered_entries
            # Should not happen.
            return UPDATE_OK, None
        if status == UPDATE_OK_NO_DATA:
            # Happens for example if the server returns 304
            return UPDATE_OK_NO_DATA, None
        # Error happened while fetching the feed.
        self._last_timestamp = None
        return UPDATE_ERROR, None

    def _fetch(self) -> tuple[str, Feed | None]:
        """Fetch GeoRSS data from external source."""
        try:
            with requests.Session() as session:
                response = session.send(self._request, timeout=10)
                if response.ok:
                    self._pre_process_response(response)
                    parser = XmlParser(self._additional_namespaces())
                    feed_data = parser.parse(response.text)
                    self.parser = parser
                    self.feed_data = feed_data
                    return UPDATE_OK, feed_data
                _LOGGER.warning(
                    "Fetching data from %s failed with status %s",
                    self._request.url,
                    response.status_code,
                )
                return UPDATE_ERROR, None
        except requests.exceptions.RequestException as request_ex:
            _LOGGER.warning(
                "Fetching data from %s failed with %s", self._request.url, request_ex
            )
            return UPDATE_ERROR, None

    def _pre_process_response(self, response):
        """Pre-process the response."""
        if response:
            _LOGGER.debug("Response encoding %s", response.encoding)
            if response.content.startswith(codecs.BOM_UTF8):
                _LOGGER.debug(
                    "UTF8 byte order mark detected, " "setting encoding to 'utf-8-sig'"
                )
                response.encoding = "utf-8-sig"

    def _filter_entries(self, entries):
        """Filter the provided entries."""
        filtered_entries = entries
        _LOGGER.debug("Entries before filtering %s", filtered_entries)
        # Always remove entries without geometry
        filtered_entries = list(
            filter(lambda entry: entry.geometry is not None, filtered_entries)
        )
        # Filter by distance.
        if self._filter_radius:
            filtered_entries = list(
                filter(
                    lambda entry: entry.distance_to_home <= self._filter_radius,
                    filtered_entries,
                )
            )
        # Filter by category.
        if self._filter_categories:
            filtered_entries = list(
                filter(
                    lambda entry: len(
                        {entry.category}.intersection(self._filter_categories)
                    )
                    > 0,
                    filtered_entries,
                )
            )
        _LOGGER.debug("Entries after filtering %s", filtered_entries)
        return filtered_entries

    def _extract_from_feed(self, feed: Feed) -> dict:
        """Extract global metadata from feed."""
        global_data: dict = {}
        author: str | None = feed.author
        if author:
            global_data[ATTR_ATTRIBUTION] = author
        return global_data

    def _extract_last_timestamp(self, feed_entries) -> datetime | None:
        """Determine latest (newest) entry from the filtered feed."""
        if feed_entries:
            dates: list[datetime] = sorted(
                [entry.published for entry in feed_entries if entry.published],
                reverse=True,
            )
            if dates:
                last_timestamp: datetime = dates[0]
                _LOGGER.debug("Last timestamp: %s", last_timestamp)
                return last_timestamp
        return None

    @property
    def last_timestamp(self) -> datetime | None:
        """Return the last timestamp extracted from this feed."""
        return self._last_timestamp



"""GeoRSS feed models."""

from __future__ import annotations

import logging

from georss_client.consts import (
    XML_TAG_COPYRIGHT,
    XML_TAG_DOCS,
    XML_TAG_ENTRY,
    XML_TAG_GENERATOR,
    XML_TAG_IMAGE,
    XML_TAG_ITEM,
    XML_TAG_LANGUAGE,
    XML_TAG_RIGHTS,
    XML_TAG_SUBTITLE,
    XML_TAG_TTL,
)
from georss_client.xml_parser.feed_image import FeedImage
from georss_client.xml_parser.feed_item import FeedItem
from georss_client.xml_parser.feed_or_feed_item import FeedOrFeedItem

_LOGGER = logging.getLogger(__name__)


class Feed(FeedOrFeedItem):
    """Represents a feed."""

    @property
    def subtitle(self) -> str | None:
        """Return the subtitle of this feed."""
        return self._attribute_with_text([XML_TAG_SUBTITLE])

    @property
    def copyright(self) -> str | None:
        """Return the copyright of this feed."""
        return self._attribute_with_text([XML_TAG_COPYRIGHT, XML_TAG_RIGHTS])

    @property
    def rights(self) -> str | None:
        """Return the rights of this feed."""
        return self.copyright

    @property
    def generator(self) -> str | None:
        """Return the generator of this feed."""
        return self._attribute_with_text([XML_TAG_GENERATOR])

    @property
    def language(self) -> str | None:
        """Return the language of this feed."""
        return self._attribute([XML_TAG_LANGUAGE])

    @property
    def docs(self) -> str | None:
        """Return the docs URL of this feed."""
        return self._attribute_with_text([XML_TAG_DOCS])

    @property
    def ttl(self) -> int | None:
        """Return the ttl of this feed."""
        return self._attribute([XML_TAG_TTL])

    @property
    def image(self) -> FeedImage | None:
        """Return the image of this feed."""
        image = self._attribute([XML_TAG_IMAGE])
        if image:
            return FeedImage(image)
        return None

    @property
    def entries(self) -> list[FeedItem]:
        """Return the entries of this feed."""
        items = self._attribute([XML_TAG_ITEM, XML_TAG_ENTRY])
        entries = []
        if items and isinstance(items, list):
            entries = [FeedItem(item) for item in items]
        else:
            # A single item in the feed is not represented as an array.
            entries.append(FeedItem(items))
        return entries



"""Tests for feed."""

import datetime
from unittest import mock

import pytest
import requests

from georss_client.consts import UPDATE_ERROR, UPDATE_OK
from georss_client.feed import GeoRssFeed
from tests import MockGeoRssFeed
from tests.utils import load_fixture

HOME_COORDINATES_1 = (-31.0, 151.0)
HOME_COORDINATES_2 = (-37.0, 150.0)


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_1.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    assert (
        repr(feed) == "<MockGeoRssFeed(home=(-31.0, 151.0), "
        "url=None, radius=None, categories=None)>"
    )
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 5

    feed_entry = entries[0]
    assert feed_entry.title == "Title 1"
    assert feed_entry.external_id == "1234"
    assert feed_entry.category == "Category 1"
    assert feed_entry.published == datetime.datetime(2018, 9, 23, 8, 30)
    assert feed_entry.updated == datetime.datetime(2018, 9, 23, 8, 35)
    assert feed_entry.coordinates == (-37.2345, 149.1234)
    assert feed_entry.distance_to_home == pytest.approx(714.4, 0.1)

    feed_entry = entries[1]
    assert feed_entry.title == "Title 2"
    assert feed_entry.external_id == "2345"
    assert feed_entry.attribution is None
    assert repr(feed_entry) == "<FeedEntry(id=2345)>"

    feed_entry = entries[2]
    assert feed_entry.title == "Title 3"
    assert feed_entry.external_id == "Title 3"

    feed_entry = entries[3]
    assert feed_entry.title is None
    assert feed_entry.external_id == hash(feed_entry.coordinates)

    feed_entry = entries[4]
    assert feed_entry.title == "Title 5"
    assert feed_entry.external_id == "5678"


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok_feed_2(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_2.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 1

    feed_entry = entries[0]
    assert feed_entry.title == "Title 1"
    assert feed_entry.external_id == "1234"
    assert feed_entry.category == "Category 1"
    assert feed_entry.coordinates == (-37.2345, 149.1234)
    assert feed_entry.distance_to_home == pytest.approx(714.4, 0.1)


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok_feed_3(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_3.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 3

    feed_entry = entries[0]
    assert feed_entry.external_id == "1234"
    assert feed_entry.coordinates == (
        pytest.approx(-34.93728111547821),
        pytest.approx(148.59710883878262),
    )
    assert feed_entry.distance_to_home == pytest.approx(491.7, 0.1)

    feed_entry = entries[1]
    assert feed_entry.external_id == "2345"
    assert feed_entry.coordinates == (-34.937170989, 148.597182317)
    assert feed_entry.distance_to_home == pytest.approx(491.8, 0.1)

    feed_entry = entries[2]
    assert feed_entry.external_id == "3456"
    assert feed_entry.coordinates == (
        pytest.approx(-29.962746645660683),
        pytest.approx(152.43090880416074),
    )
    assert feed_entry.distance_to_home == pytest.approx(176.5, 0.1)


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok_feed_6(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_6.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 1

    feed_entry = entries[0]
    assert feed_entry.title == "Title 1"
    assert feed_entry.external_id == "1234"
    assert feed_entry.category == "Category 1"
    assert feed_entry.coordinates == (-37.2345, 149.1234)
    assert feed_entry.distance_to_home == pytest.approx(714.4, 0.1)


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok_with_radius_filtering(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_1.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_2, None, filter_radius=90.0)
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 4
    assert entries[0].distance_to_home == pytest.approx(82.0, 0.1)
    assert entries[1].distance_to_home == pytest.approx(77.0, 0.1)
    assert entries[2].distance_to_home == pytest.approx(84.6, 0.1)


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_ok_with_radius_and_category_filtering(mock_session, mock_request):
    """Test updating feed is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("generic_feed_1.xml")
    )

    feed = MockGeoRssFeed(
        HOME_COORDINATES_2,
        None,
        filter_radius=90.0,
        filter_categories=["Category 2"],
    )
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 1
    assert entries[0].distance_to_home == pytest.approx(77.0, 0.1)

    feed = MockGeoRssFeed(
        HOME_COORDINATES_2,
        None,
        filter_radius=90.0,
        filter_categories=["Category 4"],
    )
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 0


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_error(mock_session, mock_request):
    """Test updating feed results in error."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = False

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    status, entries = feed.update()
    assert status == UPDATE_ERROR


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_with_request_exception(mock_session, mock_request):
    """Test updating feed raises exception."""
    mock_session.return_value.__enter__.return_value.send.side_effect = (
        requests.exceptions.RequestException
    )

    feed = GeoRssFeed(HOME_COORDINATES_1, None)
    status, entries = feed.update()
    assert status == UPDATE_ERROR
    assert entries is None


@mock.patch("requests.Request")
@mock.patch("requests.Session")
def test_update_bom(mock_session, mock_request):
    """Test updating feed with BOM (byte order mark) is ok."""
    mock_session.return_value.__enter__.return_value.send.return_value.ok = True
    mock_session.return_value.__enter__.return_value.send.return_value.text = (
        load_fixture("xml_parser_bom_1.xml")
    )

    feed = MockGeoRssFeed(HOME_COORDINATES_1, None)
    assert (
        repr(feed) == "<MockGeoRssFeed(home=(-31.0, 151.0), "
        "url=None, radius=None, categories=None)>"
    )
    status, entries = feed.update()
    assert status == UPDATE_OK
    assert entries is not None
    assert len(entries) == 0
